#include "MainWindow.h"
#include "resource.h"
#include <fstream>
#include <list>

MainWindowClass::MainWindowClass() :
   CurrentObject(DrawLineCmd),
   Color(RGB(255, 0, 0)),
   WasDown(false),
   Toolbar(0),
   Statusbar(0)
{
   createWindow();
   setText( _T("SmartWin - Draw"));
   onSized(&MainWindowClass::IsResized);

   Menu = createMenu();
   WidgetMenuPtr file = Menu->appendPopup( _T("&File") );
   file->appendItem(LoadCmd, _T("&Load File"), &MainWindowClass::MenuClicked );
   file->appendItem(SaveCmd, _T("&Save File"), &MainWindowClass::MenuClicked );
   WidgetMenuPtr help = Menu->appendPopup( _T("&Help") );
   help->appendItem(AboutCmd, _T("&About Draw..."), &MainWindowClass::MenuClicked );

   Menu->attach(this);

   setSize(100, 100, 800, 600);
   onPainting(&MainWindowClass::OnPaint);
   onLeftMouseDown(&MainWindowClass::OnLeftMouseDown);
   onLeftMouseUp(&MainWindowClass::OnLeftMouseUp);
   onMouseMove(&MainWindowClass::OnMouseMove);

   Toolbar = createToolbar();
   Toolbar->setButtonSize(16, 16);
   Toolbar->addBitmap(IDB_BITMAP1, 5);
   Toolbar->addButton(DrawRectangleCmd, 0, _T(""), _T("Rectangle"), &MainWindowClass::ToolbarClicked);
   Toolbar->addButton(DrawLineCmd,      1, _T(""), _T("Line"), &MainWindowClass::ToolbarClicked);
   Toolbar->addButton(DrawRedCmd,       2, _T(""), _T("Red"), &MainWindowClass::ToolbarClicked);
   Toolbar->addButton(DrawGreenCmd,     3, _T(""), _T("Green"), &MainWindowClass::ToolbarClicked);
   Toolbar->addButton(DrawBlueCmd,      4, _T(""), _T("Blue"), &MainWindowClass::ToolbarClicked);

   Statusbar = createStatusBarSections();
   std::vector<unsigned>Sections;
   Sections.push_back(200);
   Sections.push_back(200);
   Sections.push_back(200);
   Statusbar->setSections(Sections);
   Statusbar->setText(_T("Red"), 1);
   Statusbar->setText(_T("Line"), 2);

   Toolbar->setVisible(false);
   Sleep(100);
   Toolbar->setVisible(true);
   SmartWin::Rectangle ClientArea();
   ToolbarHeight =  Toolbar->getSizeClientArea().size.y;
}

MainWindowClass::~MainWindowClass()
{
   std::list<ObjectBaseClass *>::iterator it;
   for(it = ObjectList.begin(); it != ObjectList.end(); ++it)
      delete *it;

   ObjectList.clear();
}

void MainWindowClass::OnPaint(SmartWin::PaintCanvas &aPaintCanvas)
{
   std::list<ObjectBaseClass *>::iterator it;
   for(it = ObjectList.begin(); it != ObjectList.end(); it++)
      (*it)->Draw(aPaintCanvas);
}

void MainWindowClass::LoadFile()
{
   std::list<ObjectBaseClass *>::iterator it;
   for(it = ObjectList.begin(); it != ObjectList.end(); ++it)
      delete *it;

   ObjectList.clear();

   WidgetLoadFile dlg = createLoadFile();
   dlg.addFilter(_T("Text files (*.drw)"), _T("*.drw"));
   dlg.addFilter(_T("All files (*.*)"), _T("*.*"));
   SmartWin::tstring filePath = dlg.showDialog();
   if(filePath.size() != 0)
   {
      std::basic_ifstream<TCHAR> file(SmartWin::AsciiGuaranteed::doConvert(filePath.c_str()).c_str());
      if(!file.good())
      {
         createMessageBox().show("Failed to open file");
         return;
      }
      int ObjType;
      while(file >> ObjType)
      {
         ObjectBaseClass *Obj;
         if(ObjType == DrawLineCmd)
            Obj = new LineClass();
         else if(ObjType == DrawRectangleCmd)
            Obj = new RectangleClass;
         else
         {
            createMessageBox().show("Unknown type");
            updateWidget();
            return;
         }
         if(!Obj->Read(file))
            createMessageBox().show("Failed to read");
         ObjectList.push_back(Obj);
      }
   }
   updateWidget();
}

void MainWindowClass::SaveFile()
{
   using namespace SmartWin;
   using namespace std;
   WidgetSaveFile dlg = createSaveFile();
   dlg.addFilter( _T("Draw Files (*.drw)"), _T("*.drw") );
   dlg.addFilter( _T("All Files (*.*)"), _T("*.*") );
   tstring filePath = dlg.showDialog();
   if(filePath.size())
   {
      basic_ofstream<TCHAR> file(SmartWin::AsciiGuaranteed::doConvert(filePath.c_str()).c_str());
      if(file.good())
      {
         std::list<ObjectBaseClass *>::iterator it;
         for(it = ObjectList.begin(); it != ObjectList.end(); ++it)
            (*it)->Write(file);
      }
   }
}

void MainWindowClass::OnLeftMouseDown(const SmartWin::MouseEventResult &aEventResult)
{
   if(CurrentObject == DrawRectangleCmd)
      ObjectList.push_back(new RectangleClass(aEventResult.pos, Color));
   else
      ObjectList.push_back(new LineClass(aEventResult.pos, Color));
   WasDown = true;
}

void MainWindowClass::OnLeftMouseUp(const SmartWin::MouseEventResult &aEventResult)
{
   if(WasDown)
   {
      SmartWin::UpdateCanvas Canvas(this);
      ObjectBaseClass *Last = ObjectList.back();
      Last->Draw(Canvas, true);
      Last->SetEnd(aEventResult.pos);
      Last->Draw(Canvas, false);
      WasDown = false;
   }
}

void MainWindowClass::OnMouseMove(const SmartWin::MouseEventResult &aEventResult)
{
   if(WasDown)
   {
      SmartWin::UpdateCanvas Canvas(this);
      ObjectBaseClass *Last = ObjectList.back();
      Last->Draw(Canvas, true);
      Last->SetEnd(aEventResult.pos);
      Last->Draw(Canvas, true);
   }
   if(Statusbar)
   {
      std::string Text;
      Text = ToString(aEventResult.pos.x) + " " + ToString(aEventResult.pos.y - ToolbarHeight);
      Statusbar->setText(Text.c_str(), 0);
   }
}

void MainWindowClass::MenuClicked( WidgetMenu *aMenu, unsigned aItem)
{
   switch(aItem)
   {
   case LoadCmd:
      LoadFile();
      break;
   case SaveCmd:
      SaveFile();
      break;
   case AboutCmd:
      About();
      break;
   }
}

void MainWindowClass::About()
{
   createMessageBox().show("SmartWin Drawing application\r\nBy Bertel Brander");
}

void MainWindowClass::ToolbarClicked(WidgetToolbar *aItem, unsigned int aId)
{
   switch(aId)
   {
   case DrawLineCmd:
      CurrentObject = DrawLineCmd;
      Statusbar->setText(_T("Line"), 2);
      break;
   case DrawRectangleCmd:
      CurrentObject = DrawRectangleCmd;
      Statusbar->setText(_T("Rectangle"), 2);
      break;
   case DrawRedCmd:
      Color = RGB(255, 0, 0);
      Statusbar->setText(_T("Red"), 1);
      break;
   case DrawBlueCmd:
      Color = RGB(0, 0, 255);
      Statusbar->setText(_T("Blue"), 1);
      break;
   case DrawGreenCmd:
      Color = RGB(0, 255, 0);
      Statusbar->setText(_T("Green"), 1);
      break;
   default:
      createMessageBox().show("Ups, wrong button");
      break;
   }
}

void MainWindowClass::IsResized(const SmartWin::WindowSizedEventResult & sz)
{
	SmartWin::Rectangle ClientArea(getSizeClientArea().size);
   if(Toolbar)
      Toolbar->setSize(ClientArea);
   if(Statusbar)
      Statusbar->refresh();
}
